#include <commctrl.h>

// ID used for the separator
#define ID_SEP 0

#define BEGIN_TOOLBAR_MAP(x) public: \
	const static int* _GetToolbarEntries(int& nButtons) { \
	static const int _entries[] = {
#define TOOLBAR_BUTTON(x) x,
#define TOOLBAR_SEPARATOR()	ID_SEP,
#define END_TOOLBAR_MAP() }; nButtons = sizeof(_entries)/sizeof(int); return _entries; }

template <class T>
class CToolbar
{
public:
	CToolbar()
	{
		m_hwndToolbar = NULL;
	}
	void CreateToolbar(HWND hParent)
	{
		InitCommonControls();
		m_hwndToolbar = CreateWindowEx( 
				0,
				TOOLBARCLASSNAME,
				NULL,
				WS_CHILD | WS_BORDER | WS_VISIBLE | TBSTYLE_TOOLTIPS,
				0, 0, 0, 0,
				hParent,
				(HMENU)IDR_TOOLBAR1,
				_Module.GetResourceInstance(),
				NULL);
		AddButtons();
	}
	void AddButtons()
	{
		T* pT = static_cast<T*>(this);
		int nButtons;
		const int* arrID = pT->_GetToolbarEntries(nButtons);
		TBBUTTON* pButton = new TBBUTTON[nButtons];
	    TBADDBITMAP tbab; 
		int			nIndex = 0;

		// Send the TB_BUTTONSTRUCTSIZE message, which is required for 
		// backward compatibility. 
	    SendMessage(m_hwndToolbar, TB_BUTTONSTRUCTSIZE, sizeof(TBBUTTON), 0); 

		// Add the bitmap containing button images to the toolbar. 
		tbab.hInst = _Module.GetResourceInstance();
		tbab.nID   = IDR_TOOLBAR1; 
		SendMessage(m_hwndToolbar, TB_ADDBITMAP, nButtons, (WPARAM)&tbab); 

		// Loop around, adding each button to the TBBUTTON array
		for (int i = 0; i < nButtons; i++)
		{
			pButton[i].fsState	 = TBSTATE_ENABLED;
			pButton[i].dwData	 = 0;
			pButton[i].iString	 = 0;
			if (arrID[i] != ID_SEP)
			{
				pButton[i].iBitmap	 = nIndex;
				pButton[i].fsStyle	 = TBSTYLE_BUTTON;
				pButton[i].idCommand = arrID[i];
				nIndex++;
			}
			else
			{
				pButton[i].fsStyle	 = TBSTYLE_BUTTON|TBSTYLE_SEP;
				pButton[i].iBitmap	 = 8;	// Width of separator
				pButton[i].idCommand = 0;
			}
		}
		// Add the buttons
	    SendMessage(m_hwndToolbar, TB_ADDBUTTONS, nButtons, (LPARAM)pButton); 
		delete [] pButton;
	}
	void DestroyToolbar()
	{
		if (m_hwndToolbar != NULL)
		{
			::DestroyWindow(m_hwndToolbar);
			m_hwndToolbar = NULL;
		}
	}
	LRESULT OnToolbarNeedText(WPARAM, LPNMHDR pnmh, BOOL&)
	{
		LPTOOLTIPTEXT lpToolTipText = (LPTOOLTIPTEXT)pnmh;
		static TCHAR szBuf[128];

		// Get the string with the same ID as the button from the resource
		LoadString(_Module.GetResourceInstance(),
	    	lpToolTipText->hdr.idFrom,
		    szBuf,
			sizeof(szBuf)*sizeof(TCHAR));
		lpToolTipText->lpszText = szBuf;

		return 0;
	}
protected:
	HWND	m_hwndToolbar;
};
